<?php
/*
Copyright (C) 2008-2012 Stephane Graber <stgraber@ubuntu.com>

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
*/

function qatracker_report_defects() {
    $items = array();
    $arg = arg();

    if(count($arg) < 4) {
        $arg[3] = "all";
    }

    $noreport[] = array(
        '#type' => 'markup',
        '#prefix' => '<p>',
        '#markup' => t("<b>There is no milestone being currently tested.</b><br />"),
        '#suffix' => '</p>',
    );

    $site = qatracker_get_current_site();
    if (!$site) {
        return $noreport;
    }

    $content = qatracker_buildreport($site, $arg[3]);
    if (!$content) {
        return $noreport;
    }

    return $content;
}

function qatracker_getbugs_for_release($siteid, $header) {
    /* 
     * Get all the bugs for all the milestones of a release 
     * for a given siteid 
     */
    $query = db_select('qatracker_build', 'b')
                ->extend('TableSort')
                ->extend('PagerDefault');
    $query->distinct();
    $query->addField('m', 'title', 'mtitle');
    $query->fields('l', array(
        'bugnumber',
        'product',
        'title'));
    $query->addField('l', 'title', 'btitle');
    $query->fields('l', array(
        'status',
        'importance',
        'assignee',
        'commentscount',
        'duplicatescount',
        'subscriberscount'
        ));
    $query->addField('m', 'id');
    $query->addExpression("CASE 
            WHEN l.importance = 'Critical' THEN 0
            WHEN l.importance = 'High' THEN 1
            WHEN l.importance = 'Medium' THEN 2
            WHEN l.importance = 'Low' THEN 3
            WHEN l.importance = 'Undecided' THEN 10
        ELSE 5 END", 'importance_order');
    $query->join('qatracker_product', 'p', 'p.id = b.productid');
    $query->join('qatracker_build_milestone', 'bm', 'bm.buildid = b.id');
    $query->join('qatracker_milestone', 'm', 'bm.milestoneid = m.id');
    $query->join('qatracker_testsuite_product', 'tp', 'tp.productid = b.productid AND tp.milestone_seriesid = m.seriesid');
    $query->join('qatracker_testsuite_testcase', 'tt', 'tt.testsuiteid = tp.testsuiteid');
    $query->join('qatracker_testcase', 't', 'tt.testcaseid = t.id');
    $query->join('qatracker_result', 'r', '(r.testcaseid = t.id AND r.buildid = b.id)');
    $query->join('qatracker_bug', 'g', 'g.resultid = r.id');
    $query->join('qatracker_launchpad_bug', 'l', 'g.bugnumber = l.originalbug');
    $query->condition('m.siteid', intval($siteid));

    $subquery = db_select('qatracker_milestone', 'm2');
    $subquery->distinct();
    $subquery->addExpression("SUBSTRING(m2.title, '^[^ ]+')");
    $or = db_or();
    $or->condition('m2.status', 0, '=');
    $or->condition('m2.status', 1, '=');
    $subquery->condition($or);
    $subquery->condition('m2.siteid', 'm.siteid');

    $query->addExpression("SUBSTRING(m.title, '^[^ ]+') IN (" . $subquery->__toString() . ')');
    $or = db_or();
    $or->condition('tt.status', 0, '=');
    $or->condition('tt.status', 2, '=');
    $query->condition($or);
    $query->condition('tt.status', array(0, 2), 'IN');
    $query->orderBy('m.id', 'DESC');
    $query->orderByHeader($header);
    return $query->execute()->fetchAll();
}


function qatracker_buildreport($site, $filter) {
    /*
     * Build the defect report
     *
     * @param $site: A site object 
     * @param $filter: Type of bugs we are interested in
     *      all: All the reports
     *      closed: Only closed reports (see $closed array below)
     *      opened: Only opened reports (see $closed array below)
     *
     * TODO:
     *      Colorize the importance column like LP
     *      Nicer milestone rows
     */
    if (!$site) {
        return false;
    }

    $closed = array("Invalid", "Fix Released", "Won't Fix", "Expired", "Opinion");

    $siteid = $site->id;
    if(!is_numeric($siteid)){
        return false;
    }

    $header = array(
        array('data' => 'Bug #'),
        array('data' => 'Title', 'field' => 'btitle', 'sort' => 'DESC'),
        array('data' => 'Affects', 'field' => 'l.product', 'sort' => 'DESC'),
        array('data' => 'Status', 'field' => 'l.status', 'sort' => 'DESC'),
        array('data' => 'Importance', 'field' => 'l.importance', 'sort' => 'DESC'),
        array('data' => 'Assignee', 'field' => 'l.assignee', 'sort' => 'DESC'),
        array('data' => 'Com.', 'field' => 'l.commentscount', 'sort' => 'DESC'),
        array('data' => 'Sub.', 'field' => 'l.subscriberscount', 'sort' => 'DESC'),
        array('data' => 'Dup.', 'field' => 'l.duplicatescount', 'sort' => 'DESC'),
    );

    $bugs = qatracker_getbugs_for_release($siteid, $header);

    $curr_milestone = "";
    $rows = array();
    foreach ($bugs as $record) {
        // Milestone changed
        if ( $curr_milestone != $record->mtitle ) {
            $curr_milestone = $record->mtitle;
            $rows[] = array(
                'data' => array( array (
                    'data' => "<b>".$curr_milestone."</b>",
                    'colspan' => 9,
                    'class' => 'rpt_tbl_milestone'
                ))
            );
        }

        // FIXME
        // Bad practice: move the filter to the query
        //
        $filter = strtolower($filter);
        if (   ($filter == 'all')
            or ($filter == 'closed' and in_array($record->status, $closed))
            or ($filter == 'opened' and !in_array($record->status, $closed))) 
        {
            $rows[] = array(
                'data' => array(
                    l($record->bugnumber,
                        'http://bugs.launchpad.net/bugs/' . $record->bugnumber),
                    $record->btitle,
                    str_replace(' (Ubuntu)', '', $record->product),
                    $record->status,
                    $record->importance,
                    str_replace('None', '-', $record->assignee),
                    intval($record->commentscount),
                    intval($record->subscriberscount),
                    intval($record->duplicatescount)
                )
            );
        }
    }

    $items = array();
    $items[] = array(
                '#theme' => 'table',
                '#header' => $header,
                '#rows'=> $rows,
                '#empty' => 'No bug reported...',
               );

    return ($items);
}

?>
